﻿using MQTTnet;
using MQTTnet.Client;

const string MqttServerName = "broker.hivemq.com";
const string MqttTopicPrefix = "Altairis.MqttDemo";

// Construct client ID
var clientId = string.Join(":", Environment.MachineName, System.Diagnostics.Process.GetCurrentProcess().Id);

// Connect to server
var mqttFactory = new MqttFactory();
var client = mqttFactory.CreateMqttClient();
var options = new MqttClientOptionsBuilder()
    .WithTcpServer(MqttServerName)
    .WithClientId(clientId)
    .WithWillTopic(MqttTopicPrefix + "/presence")
    .WithWillPayload($"{clientId} Disconnected (will)")
    .Build();

Console.Write($"Connecting to {MqttServerName}...");
var cr = await client.ConnectAsync(options);
if (cr.ResultCode == MqttClientConnectResultCode.Success) {
    Console.WriteLine("OK");
} else {
    Console.WriteLine("Failed!");
    Console.WriteLine(cr.ResultCode);
}

// Send message announcing presence
Console.Write("Announcing presence...");
var presenceMessage = new MqttApplicationMessageBuilder()
    .WithTopic(MqttTopicPrefix + "/presence")
    .WithPayload($"{clientId} Connected")
        .Build();
var pr = await client.PublishAsync(presenceMessage);
if (pr.IsSuccess) {
    Console.WriteLine("OK");
} else {
    Console.WriteLine(pr.ReasonCode);
}

// Send messages to chat
while (true) {
    // Read message from console
    Console.Write("> ");
    var chatText = Console.ReadLine();

    // Exit on empty message
    if (string.IsNullOrEmpty(chatText)) break;

    // Send message
    Console.Write("Sending...");
    var chatMessage = new MqttApplicationMessageBuilder()
        .WithTopic(MqttTopicPrefix + "/chat")
        .WithPayload($"{clientId} {chatText}")
        .Build();
    pr = await client.PublishAsync(chatMessage);
    if (pr.IsSuccess) {
        Console.WriteLine("OK");
    } else {
        Console.WriteLine(pr.ReasonCode);
    }
}

// Disconnect from server with farewell message
Console.Write($"Disconnecting from {MqttServerName}...");
presenceMessage = new MqttApplicationMessageBuilder()
    .WithTopic(MqttTopicPrefix + "/presence")
    .WithPayload($"{clientId} Disconnected")
        .Build();
await client.PublishAsync(presenceMessage);
await client.TryDisconnectAsync();
Console.WriteLine("OK");