﻿using MQTTnet;
using MQTTnet.Client;

const string MqttServerName = "broker.hivemq.com";
const string MqttTopicPrefix = "Altairis.MqttDemo";

// Construct client ID
var clientId = string.Join(":", Environment.MachineName, System.Diagnostics.Process.GetCurrentProcess().Id);

// Connect to server
var mqttFactory = new MqttFactory();
var client = mqttFactory.CreateMqttClient();
var options = new MqttClientOptionsBuilder()
    .WithTcpServer(MqttServerName)
    .WithClientId(clientId)
    .Build();

Console.Write($"Connecting to {MqttServerName}...");
var cr = await client.ConnectAsync(options);
if (cr.ResultCode == MqttClientConnectResultCode.Success) {
    Console.WriteLine("OK");
} else {
    Console.WriteLine("Failed!");
    Console.WriteLine(cr.ResultCode);
}

// Set handler for incoming message
client.ApplicationMessageReceivedAsync += evt => {
    Console.WriteLine($"{evt.ApplicationMessage.Topic}> {evt.ApplicationMessage.ConvertPayloadToString()}");
    return Task.CompletedTask;
};

// Subscribe for all messages under our prefix
var mqttSubscribeOptions = mqttFactory.CreateSubscribeOptionsBuilder()
    .WithTopicFilter(f => { f.WithTopic(MqttTopicPrefix + "/#"); })
    .Build();
await client.SubscribeAsync(mqttSubscribeOptions, CancellationToken.None);

// Wait until keypress
Console.WriteLine("Waiting for messages. Press ENTER to exit.");
Console.ReadLine();

// Disconnect from server with farewell message
Console.Write($"Disconnecting from {MqttServerName}...");
await client.TryDisconnectAsync();
Console.WriteLine("OK");